"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MnemonicBIP39 = void 0;
const bip_0039_en_json_1 = __importDefault(require("./bip-0039-en.json"));
const utlis_1 = require("./utlis");
class MnemonicBIP39 {
    constructor(mnemonic = [], options) {
        if (mnemonic.length && mnemonic.length !== 24) {
            throw new Error('Mnemonic: must contain 24 bip39 words.');
        }
        if (mnemonic.length && !mnemonic.every(word => bip_0039_en_json_1.default.includes(word))) {
            throw new Error('Mnemonic: invalid mnemonic phrase words.');
        }
        const { salt = '', rounds = 2048, keyLength = 64 } = options || {};
        const words = mnemonic.length ? mnemonic : (0, utlis_1.generateWords)();
        const seed = (0, utlis_1.generateSeed)(words, this.generateSalt(salt), rounds, keyLength).slice(0, 32);
        const keys = (0, utlis_1.generateKeyPair)(seed);
        this._words = words;
        this._seed = seed;
        this._keys = keys;
    }
    get words() {
        return this._words;
    }
    get seed() {
        return this._seed;
    }
    get keys() {
        return this._keys;
    }
    static generateWords() {
        return (0, utlis_1.generateWords)();
    }
    static generateKeyPair(seed) {
        return (0, utlis_1.generateKeyPair)(seed);
    }
    static generateSeed(mnemonic, salt = null, rounds = 2048, keyLength = 64) {
        const _salt = 'mnemonic' + (salt !== null ? (0, utlis_1.normalize)(salt) : '');
        const seed = (0, utlis_1.generateSeed)(mnemonic, _salt, rounds, keyLength);
        return seed.slice(0, 32);
    }
    static async generateSeedAsync(mnemonic, salt = null, rounds = 2048, keyLength = 64) {
        const _salt = 'mnemonic' + (salt !== null ? (0, utlis_1.normalize)(salt) : '');
        const seed = await (0, utlis_1.generateSeedAsync)(mnemonic, _salt, rounds, keyLength);
        return seed.slice(0, 32);
    }
    generateSalt(salt = null) {
        return 'mnemonic' + (salt !== null ? (0, utlis_1.normalize)(salt) : '');
    }
}
exports.MnemonicBIP39 = MnemonicBIP39;
//# sourceMappingURL=mnemonic-bip39.js.map